module TreasureHunt
  require_relative 'player'
  require_relative 'die'
  require_relative 'game_turn'
  require_relative 'treasure_hunt'

  class Game
    attr_reader :title

    def initialize(title)
      @title = title.capitalize
      @players = []
    end

    def high_score_entry(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end

    def save_high_scores(to_file='highscores.txt')
      File.open('highscores.txt', 'w') do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each { |p| file.puts high_score_entry(p) }
      end
    end

    def load_players(from_file)
      File.foreach(from_file) { |line| add_player(Player.new(line.chomp)) }
    end

    def add_player(p)
      @players << p
    end

    def total_points
      @players.reduce(0) { |memo, object| memo += object.points }
    end

    def print_stats
      puts "\n#{@title} statistics:"

      @players.sort.each do |p|
        puts "\n#{p.name}'s points totals:"
        p.each_found_treasure do |t|
          puts "#{t.points} total #{t.name} points"
        end
        puts "#{p.points} grand total points"
      end

      puts "\n#{total_points} total points from treasures found"

      strong_players, weak_players = @players.partition { |p| p.strong? }

      puts "\n#{strong_players.size} strong players:"
      strong_players.sort.each { |p| puts p }

      puts "\n#{weak_players.size} weak players:"
      weak_players.sort.each { |p| puts p }

      puts "\n#{@title} High Scores:"
      @players.sort.each { |p| puts high_score_entry(p) }
    end

    def play(rounds)
      treasures = TreasureHunt::TREASURES

      puts "\nThere are #{treasures.length} treasures to be found:"
      treasures.each { |t| puts "A #{t.name} is worth #{t.points} points" }

      puts "\nThere are #{@players.length} players in the game."

      1.upto(rounds) do |r|
        # break if yield if block_given?
        puts "\nRound #{r}:"
        @players.each do |p|
          GameTurn.take_turn(p)
        end
      end
    end
  end
end
